/*-
 * Copyright (c) 2009 - 2015 Rozhuk Ivan <rozhuk.im@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Rozhuk Ivan <rozhuk.im@gmail.com>
 *
 */

#pragma comment(linker, "/subsystem:Windows,5")
#pragma comment(linker, "/version:5")

#ifdef NDEBUG
	//#pragma comment(linker, "/MERGE:.rdata=.text")
	//#pragma comment(linker, "/FILEALIGN:512 /IGNORE:4078")
	//#pragma comment(linker, "/opt:nowin98")
	#pragma comment(linker, "/nodefaultlib")
	#pragma comment(linker, "/ENTRY:WinMainEx")
#endif //_DEBUG


#define WINVER       0x501
#define _WIN32_WINNT 0x501
//#define WINVER       0x600
//#define _WIN32_WINNT 0x600
//#define UNICODE
//#define _UNICODE


// Windows Header Files:
#include <windows.h>
#include <Shellapi.h>
#include <Ras.h>
#include <RasError.h>
#include <Rasdlg.h>
#ifdef NDEBUG
	#include "minicrt.h"
#endif //_DEBUG*/
#include "CRCFileCheck.h"
#include "ReportError.h"
#include "Shortcut.h"


#pragma comment(lib, "Rasapi32.lib")
#pragma comment(lib, "Rasdlg.lib")
#pragma comment(lib, "Shell32.lib")


#define MEMALLOC(Size)		HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (Size + sizeof(SIZE_T)))
#define MEMREALLOC(Mem, Size)	HeapReAlloc(GetProcessHeap(), (HEAP_ZERO_MEMORY), (LPVOID)Mem, (Size + sizeof(SIZE_T)))
#define MEMFREE(Mem)		if (Mem) { HeapFree(GetProcessHeap(), 0, (LPVOID)Mem); Mem = NULL; }
#define GetSpecialFolderPath(nFolder, pszPath) SUCCEEDED(SHGetFolderPath(NULL, nFolder, NULL, SHGFP_TYPE_CURRENT, pszPath))
#define SIZEOF(X)		(sizeof(X) / sizeof(X[0]))

#define MAX_PATH_LEN		32768
LPCTSTR lpcszMainService = TEXT("RasMan");


// conn file
LPCTSTR lpcszConnSectionName = TEXT("Connection");
LPCTSTR lpcszConnName = TEXT("ConnectionName");
LPCTSTR lpcszRasEntryName = TEXT("RasEntry");


DWORD CheckServicesFor(LPCWSTR lpServiceName);
DWORD IsSomeDependenciesFailed(SC_HANDLE hSCManager, SC_HANDLE hService);

DWORD StartServicesFor(LPCWSTR lpServiceName);
DWORD StartDependenciesServices(SC_HANDLE hSCManager, SC_HANDLE hService);

BOOL IsFileExist(LPCTSTR lpFileName);



#ifdef _DEBUG
	#ifdef UNICODE
		int APIENTRY wWinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPWSTR lpCmdLine,int nCmdShow)
	#else
		int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
	#endif
#else
	void WinMainEx(void)
#endif //_DEBUG
{
	#ifdef NDEBUG
		HINSTANCE	hInstance = GetModuleHandle(NULL);
		HINSTANCE	hPrevInstance = NULL;
		LPTSTR		lpCmdLine = GetCommandLine();
		int		nCmdShow = 0;
	#endif //_DEBUG

	DWORD dwRet = NO_ERROR;

	//    
	dwRet = CRCFileCheck();
	if (dwRet == NO_ERROR) { //     
		BOOL bErrNotified = FALSE;
		DWORD dwSelfFileName;
		WCHAR szEntryName[RAS_MaxEntryName + 1], szConnFileName[MAX_PATH_LEN];
		RASENTRY reEntry = { 0 };

		dwSelfFileName = GetModuleFileName(NULL, szConnFileName, MAX_PATH_LEN);
		if (dwSelfFileName) { // Set CurDir to this program location
			WCHAR szSelfWorkDirName[MAX_PATH_LEN];
			LPWSTR lpwszSelfFileName;
			SIZE_T dwSelfWorkDirLen;

	#ifdef NDEBUG
			lpCmdLine += (dwSelfFileName + 3);
	#endif //_DEBUG
			dwSelfWorkDirLen = GetFullPathName(szConnFileName, MAX_PATH_LEN, szSelfWorkDirName, &lpwszSelfFileName);
			if (dwSelfWorkDirLen) {
				dwSelfWorkDirLen -= lstrlen(lpwszSelfFileName);
				szSelfWorkDirName[dwSelfWorkDirLen] = 0;
				SetCurrentDirectory(szSelfWorkDirName);
			}
			szConnFileName[0] = 0;
		}

		if (lpCmdLine && lstrlen(lpCmdLine)) {
			int iNumArgs;
			LPWSTR *lpargv;

			lpargv = CommandLineToArgvW(lpCmdLine, &iNumArgs);
			if (lpargv) {
				//if (IsFileExist(lpCmdLine))
				{
				//	lstrcpyn(szConnFileName, lpCmdLine, MAX_PATH_LEN);
				//} else {
					GetFullPathName(lpargv[0], MAX_PATH_LEN, szConnFileName, NULL);
				}
				LocalFree(lpargv);
			}
		}

		if (IsFileExist(szConnFileName) == FALSE) {
			DWORD dwConnFileNameLen;

			dwConnFileNameLen = GetModuleFileName(NULL, szConnFileName, MAX_PATH_LEN);
			if (dwConnFileNameLen)
				(*((DWORDLONG*)((LPWSTR)szConnFileName + (dwConnFileNameLen - 4)))) = (*((DWORDLONG*)(TEXT(".ini"))));
		}

		if (IsFileExist(szConnFileName)) {
			dwRet = ERROR_FILE_CORRUPT;
			if (GetPrivateProfileStruct(lpcszConnSectionName, lpcszRasEntryName,
			    &reEntry, sizeof(RASENTRY), szConnFileName)) {
				if (reEntry.dwSize == sizeof(RASENTRY)) {
					GetPrivateProfileString(lpcszConnSectionName, lpcszConnName,
					    reEntry.szDeviceType, szEntryName, SIZEOF(szEntryName), szConnFileName);
					//reEntry.dwSize = sizeof(RASENTRY);
					dwRet = NO_ERROR;
				}
			}
		} else { // predefined settings
#if 1 /* Enable buildin connection */
			lstrcpyn(szEntryName, TEXT(" - "), SIZEOF(szEntryName));

			reEntry.dwSize = sizeof(RASENTRY);
			lstrcpyn(reEntry.szLocalPhoneNumber, TEXT("172.16.0.254"), RAS_MaxPhoneNumber);
			reEntry.dwfNetProtocols = RASNP_Ip;
			reEntry.dwFramingProtocol = RASFP_Ppp;
			reEntry.dwfOptions = RASEO_IpHeaderCompression | RASEO_RemoteDefaultGateway | RASEO_ModemLights | RASEO_SwCompression | RASEO_PreviewUserPw | RASEO_ShowDialingProgress | RASEO_Custom | RASEO_RequireSPAP | RASEO_RequireCHAP | RASEO_RequireMsCHAP | RASEO_RequireMsCHAP2 | RASEO_RequireW95MSCHAP;
			reEntry.dwfOptions2 = RASEO2_SecureFileAndPrint | RASEO2_SecureClientForMSNet | RASEO2_Internet | RASEO2_DisableNbtOverIP | RASEO2_ReconnectIfDropped;
			lstrcpyn(reEntry.szDeviceType, RASDT_Vpn, RAS_MaxDeviceType);
			reEntry.dwType = RASET_Vpn;
			reEntry.dwVpnStrategy = VS_PptpOnly;
			reEntry.dwEncryptionType = ET_Optional;
			reEntry.dwRedialCount = 9999999;
			reEntry.dwRedialPause = 3;
			dwRet = NO_ERROR;
#else
			dwRet=ERROR_FILE_NOT_FOUND;
#endif
		}


		if (dwRet == NO_ERROR) { // szEntryName and reEntry valid
			dwRet = CheckServicesFor(lpcszMainService);
			if (dwRet == ERROR_SERVICE_DISABLED) {
				if (MessageBox(NULL,
				    TEXT("        .\r\n  ?"),
				    NULL, (MB_YESNO | MB_DEFBUTTON1 | MB_ICONWARNING)) == IDYES) {
					dwRet = StartServicesFor(lpcszMainService);
					if (dwRet == ERROR_ACCESS_DENIED) {
						MessageBox(NULL,
						    TEXT("          .\r\n               ."),
						    NULL, (MB_OK | MB_ICONERROR));
						bErrNotified = TRUE;
					}
				} else {
					bErrNotified = TRUE;
				}
			}

			if (dwRet == NO_ERROR) {
				//if (RasValidateEntryName(NULL, szEntryName) == NO_ERROR)//ERROR_ALREADY_EXISTS//ERROR_INVALID_NAME
				dwRet = RasSetEntryProperties(NULL, szEntryName, &reEntry, sizeof(RASENTRY), NULL, 0);
				if (dwRet == NO_ERROR) { //  /
					RASDIALDLG rddDialDlg = { 0 };

					if (TRUE) { //     
						WCHAR szPath[MAX_PATH_LEN], wszBuff[1025];

						if (GetSpecialFolderPath(CSIDL_DESKTOPDIRECTORY, szPath))
						{
							wsprintf(wszBuff, TEXT("%s\\%s.lnk"), szPath, szEntryName);
							CreateShortcutToConn(wszBuff, szEntryName);

							//wsprintf(wszBuff, TEXT("%s\\ .url"), szPath);
							//CreateShortcutToURL(wszBuff, L"https://secretplace.ru/");
						}
					}

					rddDialDlg.dwSize = sizeof(RASDIALDLG);
					if (RasDialDlg(NULL, szEntryName, NULL, &rddDialDlg) == FALSE)
						dwRet = rddDialDlg.dwError;
				}
			} else { //   ,      
				if (bErrNotified)
					dwRet = NO_ERROR;
			}
		}
	}

	if (dwRet != NO_ERROR)
		ReportError(dwRet);

#ifdef _DEBUG
	return(dwRet);
#else
	ExitProcess(dwRet);
#endif //_DEBUG
}


DWORD
CheckServicesFor(LPCWSTR lpServiceName) {
	DWORD dwRet;
	SC_HANDLE hSCManager, hService;

	hSCManager = OpenSCManager(NULL,SERVICES_ACTIVE_DATABASE,SC_MANAGER_CONNECT);
	if (!hSCManager)
		return (GetLastError());
	hService = OpenService(hSCManager, lpServiceName, (SERVICE_QUERY_CONFIG | SERVICE_QUERY_STATUS));
	if (hService) {
		dwRet = IsSomeDependenciesFailed(hSCManager, hService);
		CloseServiceHandle(hService);
	} else {
		dwRet = GetLastError();
	}
	CloseServiceHandle(hSCManager);

	return(dwRet);
}


DWORD
IsSomeDependenciesFailed(SC_HANDLE hSCManager, SC_HANDLE hService) {
	DWORD dwRet = NO_ERROR;
	SIZE_T cbBytesNeeded = 0;
	SERVICE_STATUS_PROCESS sspProcessStatus;
	QUERY_SERVICE_CONFIG *pqscServiceConfig;

	if (!QueryServiceStatusEx(hService, SC_STATUS_PROCESS_INFO,
	    (LPBYTE)&sspProcessStatus, sizeof(SERVICE_STATUS_PROCESS), &cbBytesNeeded))
		return (GetLastError());
	if (sspProcessStatus.dwCurrentState == SERVICE_RUNNING)
		return (NO_ERROR);
	//   ,     
	QueryServiceConfig(hService, NULL, 0, &cbBytesNeeded);
	dwRet = GetLastError();
	if (dwRet != ERROR_INSUFFICIENT_BUFFER)
		return (dwRet);
	//       (  )
	pqscServiceConfig = (QUERY_SERVICE_CONFIG*)MEMALLOC(cbBytesNeeded);
	if (!pqscServiceConfig)
		return (GetLastError());
	if (QueryServiceConfig(hService, pqscServiceConfig, cbBytesNeeded, &cbBytesNeeded)) {
		if (pqscServiceConfig->dwStartType == SERVICE_DISABLED)	{ //   :(
			dwRet = ERROR_SERVICE_DISABLED;
		} else { //  ,      
			dwRet = NO_ERROR;
			if (pqscServiceConfig->lpDependencies) {
				LPWSTR lpszCurrServiceName = pqscServiceConfig->lpDependencies;
				SIZE_T dwCurrServiceNameLen;
				SC_HANDLE hCurrService;

				while ((dwCurrServiceNameLen = lstrlen(lpszCurrServiceName)) && dwRet == NO_ERROR) {
					//   ,     
					hCurrService = OpenService(hSCManager, lpszCurrServiceName,
					    (SERVICE_QUERY_CONFIG | SERVICE_QUERY_STATUS));
					if (hCurrService) {
						dwRet = IsSomeDependenciesFailed(hSCManager, hCurrService);
						CloseServiceHandle(hCurrService);
					}
					lpszCurrServiceName += (dwCurrServiceNameLen + 1);
				}
			}
		}
	} else {
		dwRet = GetLastError();
	}
	MEMFREE(pqscServiceConfig);

	return (dwRet);
}



DWORD
StartServicesFor(LPCWSTR lpServiceName) {
	DWORD dwRet;
	SC_HANDLE hSCManager, hService;

	hSCManager = OpenSCManager(NULL, SERVICES_ACTIVE_DATABASE, SC_MANAGER_CONNECT);
	if (!hSCManager)
		return (GetLastError());
	hService = OpenService(hSCManager, lpServiceName,
	    (SERVICE_QUERY_CONFIG | SERVICE_QUERY_STATUS | SERVICE_START | SERVICE_CHANGE_CONFIG));
	if (hService) {
		dwRet = StartDependenciesServices(hSCManager, hService);
		CloseServiceHandle(hService);
	} else {
		dwRet = GetLastError();
	}
	CloseServiceHandle(hSCManager);

	return(dwRet);
}


DWORD
StartDependenciesServices(SC_HANDLE hSCManager, SC_HANDLE hService) {
	DWORD dwRet = NO_ERROR;
	SIZE_T cbBytesNeeded = 0;
	SERVICE_STATUS_PROCESS sspProcessStatus;

	if (!QueryServiceStatusEx(hService, SC_STATUS_PROCESS_INFO,
	    (LPBYTE)&sspProcessStatus, sizeof(SERVICE_STATUS_PROCESS), &cbBytesNeeded))
		return (GetLastError());
	if (sspProcessStatus.dwCurrentState == SERVICE_RUNNING)
		return (NO_ERROR);
	//   ,     
	QueryServiceConfig(hService, NULL, 0, &cbBytesNeeded);
	dwRet = GetLastError();
	if (dwRet != ERROR_INSUFFICIENT_BUFFER)
		return (dwRet);
	//       (  )
	QUERY_SERVICE_CONFIG *pqscServiceConfig;

	pqscServiceConfig = (QUERY_SERVICE_CONFIG*)MEMALLOC(cbBytesNeeded);
	if (!pqscServiceConfig)
		return (GetLastError());
	if (QueryServiceConfig(hService, pqscServiceConfig, cbBytesNeeded, &cbBytesNeeded)) {
		//      
		dwRet = NO_ERROR;
		if (pqscServiceConfig->lpDependencies) {
			LPWSTR lpszCurrServiceName = pqscServiceConfig->lpDependencies;
			SIZE_T dwCurrServiceNameLen;
			SC_HANDLE hCurrService;

			while ((dwCurrServiceNameLen = lstrlen(lpszCurrServiceName)) && dwRet == NO_ERROR) {
				//   ,     
				hCurrService = OpenService(hSCManager, lpszCurrServiceName,
				    (SERVICE_QUERY_CONFIG | SERVICE_QUERY_STATUS | SERVICE_START | SERVICE_CHANGE_CONFIG));
				if (hCurrService) {
					dwRet = StartDependenciesServices(hSCManager, hCurrService);
					CloseServiceHandle(hCurrService);
				}
				lpszCurrServiceName += (dwCurrServiceNameLen + 1);
			}
		}

		if (dwRet == NO_ERROR) { //      ,  
			if (pqscServiceConfig->dwStartType == SERVICE_DISABLED) { //   :(
				if (ChangeServiceConfig(hService, SERVICE_NO_CHANGE, SERVICE_AUTO_START,
				    SERVICE_NO_CHANGE, NULL, NULL, NULL, NULL, NULL, NULL, NULL) == FALSE) {
					dwRet = GetLastError();
				}
			}

			if (dwRet == NO_ERROR) {
				if (StartService(hService, 0, NULL)) {
					dwRet = NO_ERROR;
				} else {
					dwRet = GetLastError();
				}
			}
		}
	} else {
		dwRet = GetLastError();
	}
	MEMFREE(pqscServiceConfig);

	return(dwRet);
}




BOOL
IsFileExist(LPCTSTR lpFileName) {
	BOOL bRet = FALSE;
	WIN32_FILE_ATTRIBUTE_DATA wfad;

	if (GetFileAttributesEx(lpFileName, GetFileExInfoStandard, (LPWIN32_FILE_ATTRIBUTE_DATA)&wfad))
		bRet = !(wfad.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
	return(bRet);
}



extern "C++" void *_memset(void *pvDst,int val,size_t count)
{
	return(memset(pvDst, val, count));
}


extern "C" __declspec(naked) void __cdecl _chkstk()
{
	#define _PAGESIZE_ 4096

	__asm{
			push    ecx

	; Calculate new TOS.

			lea     ecx, [esp] + 8 - 4      ; TOS before entering function + size for ret value
			sub     ecx, eax                ; new TOS

	; Handle allocation size that results in wraparound.
	; Wraparound will result in StackOverflow exception.

			sbb     eax, eax                ; 0 if CF==0, ~0 if CF==1
			not     eax                     ; ~0 if TOS did not wrapped around, 0 otherwise
			and     ecx, eax                ; set to 0 if wraparound

			mov     eax, esp                ; current TOS
			and     eax, not ( _PAGESIZE_ - 1) ; Round down to current page boundary

	cs10:
			cmp     ecx, eax                ; Is new TOS
			jb      short cs20              ; in probed page?
			mov     eax, ecx                ; yes.
			pop     ecx
			xchg    esp, eax                ; update esp
			mov     eax, dword ptr [eax]    ; get return address
			mov     dword ptr [esp], eax    ; and put it at new TOS
			ret

	; Find next lower page and probe
	cs20:
			sub     eax, _PAGESIZE_         ; decrease by PAGESIZE
			test    dword ptr [eax],eax     ; probe page.
			jmp     short cs10
	}
}//

