/*
 * Copyright (c) 2009 Rozhuk Ivan <rozhuk.im@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */



#if !defined(AFX_SHORTCUT__H__INCLUDED_)
#define AFX_SHORTCUT__H__INCLUDED_



#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <intshcut.h>


#pragma comment(lib,"shlwapi.lib")


#define HOTKEY(modifier,key)	((((modifier)&0xff)<<8)|((key)&0xff)) 




BOOL ShellCreateShortcutFile(LPCWSTR pwzShortCutFileName,LPCWSTR pszPathAndFileName,LPCWSTR pszWorkingDirectory,LPCWSTR pszArguments,WORD wHotKey,int iCmdShow,LPCWSTR pszIconFileName,int iIconIndex)
{
	IShellLink *pShortcut;//  
	IPersistFile *pShortcutFile;//,      
	HRESULT hRes;//    ""

	CoInitialize(NULL);
	hRes=CoCreateInstance(CLSID_ShellLink,0,CLSCTX_INPROC_SERVER,IID_IShellLink,(LPVOID*)&pShortcut);
	if(SUCCEEDED(hRes))
	{
		hRes=pShortcut->SetPath(pszPathAndFileName);
		if(SUCCEEDED(hRes))
		{
			hRes=pShortcut->SetArguments(pszArguments);
			if(SUCCEEDED(hRes))
			{
				hRes=pShortcut->SetWorkingDirectory(pszWorkingDirectory);
				if(SUCCEEDED(hRes))
				{
					hRes=pShortcut->SetIconLocation(pszIconFileName,iIconIndex);
					if(SUCCEEDED(hRes))
					{
						hRes=pShortcut->SetHotkey(wHotKey);
						if(SUCCEEDED(hRes))
						{
							hRes=pShortcut->SetShowCmd(iCmdShow);
							if(SUCCEEDED(hRes))
							{//    
								hRes = pShortcut->QueryInterface(IID_IPersistFile,(LPVOID*)&pShortcutFile);
								if(SUCCEEDED(hRes))
								{//   
									hRes=pShortcutFile->Save(pwzShortCutFileName,TRUE);
									pShortcutFile->Release();
								}
							}
						}
					}
				}
			}
		}
		pShortcut->Release();
	}
	CoFreeUnusedLibraries();
	CoUninitialize();
return(SUCCEEDED(hRes));
}


BOOL ShellCreateShortcutObject(LPCWSTR pwzShortCutFileName,PCIDLIST_ABSOLUTE pidl,LPCWSTR pszWorkingDirectory,LPCWSTR pszArguments,WORD wHotKey,int iCmdShow,LPCWSTR pszIconFileName,int iIconIndex)
{
	IShellLink *pShortcut;// 
	IPersistFile *pShortcutFile;//,      
	HRESULT hRes;

	CoInitialize(NULL);
	hRes=CoCreateInstance(CLSID_ShellLink,0,CLSCTX_INPROC_SERVER,IID_IShellLink,(LPVOID*)&pShortcut);//    ""
	if(SUCCEEDED(hRes))
	{
		hRes=pShortcut->SetIDList(pidl);
		if(SUCCEEDED(hRes))
		{
			hRes=pShortcut->SetArguments(pszArguments);
			if(SUCCEEDED(hRes))
			{
				hRes=pShortcut->SetWorkingDirectory(pszWorkingDirectory);
				if(SUCCEEDED(hRes))
				{
					hRes=pShortcut->SetIconLocation(pszIconFileName,iIconIndex);
					if(SUCCEEDED(hRes))
					{
						hRes=pShortcut->SetHotkey(wHotKey);
						if(SUCCEEDED(hRes))
						{
							hRes=pShortcut->SetShowCmd(iCmdShow);
							if(SUCCEEDED(hRes))
							{//    
								hRes=pShortcut->QueryInterface(IID_IPersistFile,(LPVOID*)&pShortcutFile);
								if(SUCCEEDED(hRes))
								{//   
									hRes=pShortcutFile->Save(pwzShortCutFileName,TRUE);
									pShortcutFile->Release();
								}
							}
						}
					}
				}
			}
		}
		pShortcut->Release();
	}
	CoFreeUnusedLibraries();
	CoUninitialize();
return(SUCCEEDED(hRes));
}


BOOL CreateShortcutToURL(LPCWSTR pszLinkFile,LPCWSTR pszURL)
{
	HRESULT hRes;
	IPersistFile *pShortcutFile=NULL;
	IUniformResourceLocator *pURL=NULL;

	// Create an IUniformResourceLocator object
	CoInitialize(NULL);
	hRes=CoCreateInstance(CLSID_InternetShortcut,NULL,CLSCTX_INPROC_SERVER,IID_IUniformResourceLocator,(LPVOID*)&pURL);
	if (SUCCEEDED(hRes))
	{
		hRes=pURL->SetURL(pszURL,0);
		if(SUCCEEDED(hRes))
		{
			hRes=pURL->QueryInterface(IID_IPersistFile,(void**)&pShortcutFile);
			if(SUCCEEDED(hRes))
			{   
				// Save the shortcut via the IPersistFile::Save member function.
				hRes=pShortcutFile->Save(pszLinkFile,TRUE);

				// Release the pointer to IPersistFile.
				pShortcutFile->Release();
			}
		}
		// Release the pointer to IUniformResourceLocator
		pURL->Release();
	}
	CoFreeUnusedLibraries();
	CoUninitialize();
return(SUCCEEDED(hRes));
}


/******************************************************************************/
/* Functions copied from http://msdn.microsoft.com */
LPITEMIDLIST GetNextItemID(LPCITEMIDLIST pidl) 
{
	LPITEMIDLIST lpiListRet=NULL;

	if(pidl)
	{// Check for valid pidl.
		if (pidl->mkid.cb)
		{// Get the size of the specified item identifier.// If the size is zero, it is the end of the list.
			pidl=(LPITEMIDLIST)(((LPBYTE)pidl)+pidl->mkid.cb);// Add cb to pidl (casting to increment by bytes). 
			lpiListRet=((pidl->mkid.cb==0)? NULL:(LPITEMIDLIST)pidl);// Return NULL if it is null-terminating, or a pidl otherwise.
		}
   }
return(lpiListRet);
} 

SIZE_T GetSize(LPCITEMIDLIST pidl)
{// Get size of PIDL
    SIZE_T dwTotal=0;
    
	if (pidl)
    {
        dwTotal+=sizeof(pidl->mkid.cb);// Terminating null character
        while(pidl)
        {
            dwTotal+=pidl->mkid.cb;
            pidl=GetNextItemID(pidl);
        }
    }
return(dwTotal);
}

LPITEMIDLIST AppendPIDL(LPMALLOC pMalloc,LPCITEMIDLIST pidlBase, LPCITEMIDLIST pidlAdd)
{
	LPITEMIDLIST lpiListRet=NULL;

	if(pidlBase)
	{
		if(pidlAdd==NULL)
		{
			lpiListRet=(LPITEMIDLIST)pidlBase;
		}else{
			LPITEMIDLIST pidlNew;

			SIZE_T cb1=(GetSize(pidlBase)-sizeof(pidlBase->mkid.cb));
			SIZE_T cb2=GetSize(pidlAdd);
			
			pidlNew=(LPITEMIDLIST)pMalloc->Alloc((cb1+cb2));
			if(pidlNew)
			{
				CopyMemory(pidlNew, pidlBase, cb1);
				CopyMemory(((LPSTR)pidlNew) + cb1, pidlAdd, cb2);
				lpiListRet=pidlNew;
			}
		}
	}
return(lpiListRet);
}


BOOL CreateShortcutToConn(LPCWSTR pwzShortCutFileName,LPCWSTR lpszConnName)
{
    HRESULT hRes;
    ULONG ulCeltFetched;
 	LPMALLOC pMalloc;
	IShellFolder *psfDeskTop,*pConnections;
    LPITEMIDLIST pidConnections,pidlItems,pidFull;
    LPENUMIDLIST ppenum=NULL;
    STRRET strCurConnName;
	WCHAR wszCurConnName[MAX_PATH]={0};

    CoInitialize(NULL);
	hRes=SHGetMalloc(&pMalloc);
	if (SUCCEEDED(hRes))
	{
		hRes=SHGetFolderLocation(NULL,CSIDL_CONNECTIONS,NULL,NULL,&pidConnections);
		if (SUCCEEDED(hRes))
		{
		    hRes=SHGetDesktopFolder(&psfDeskTop);
			if (SUCCEEDED(hRes))
			{
				hRes=psfDeskTop->BindToObject(pidConnections,NULL,IID_IShellFolder,(LPVOID*)&pConnections);
				if (SUCCEEDED(hRes))
				{
					hRes=pConnections->EnumObjects(NULL,(SHCONTF_FOLDERS|SHCONTF_NONFOLDERS),&ppenum);
					if (SUCCEEDED(hRes))
					{// Loop for searching our connection
						while((hRes=ppenum->Next(1,&pidlItems,&ulCeltFetched))==S_OK && ulCeltFetched==1)
						{
							pConnections->GetDisplayNameOf(pidlItems,SHGDN_INFOLDER,&strCurConnName);
							StrRetToBuf(&strCurConnName,pidlItems,wszCurConnName,MAX_PATH);
							if(CompareString(MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),NORM_IGNORECASE,lpszConnName,-1,wszCurConnName,-1)==CSTR_EQUAL)
							{
								pidFull=AppendPIDL(pMalloc,pidConnections,pidlItems);
								if(pidFull)
								{
									ShellCreateShortcutObject(pwzShortCutFileName,pidFull,NULL,NULL,0,SW_SHOWNORMAL,NULL,0);
									pMalloc->Free(pidFull);
								}
								break;
							}
						}
						CoTaskMemFree(pidlItems);//pMalloc->Free(pidlItems);
						ppenum->Release();
					}
					pConnections->Release();
				}
				psfDeskTop->Release();
			}
			ILFree(pidConnections);//pMalloc->Free(pidConnections);
		}
		pMalloc->Release();
	}
	CoFreeUnusedLibraries();
	CoUninitialize();
return(SUCCEEDED(hRes));
}








#endif // !defined(AFX_SHORTCUT__H__INCLUDED_)
