/*
 * Copyright (c) 2009 Rozhuk Ivan <rozhuk.im@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */




#if !defined(AFX_MEM_FUNCS__H__INCLUDED_)
#define AFX_MEM_FUNCS__H__INCLUDED_



#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>


#if defined (_M_IA64) || defined (_M_AMD64)
//http://msdn.microsoft.com/en-us/library/ywwcatsa.aspx
	#include <intrin.h>

	#pragma intrinsic(__stosb)
	#pragma intrinsic(__movsb)

#endif

extern "C" void * __cdecl memmove(void *pvDst, const void *pvSrc, size_t count);
#pragma intrinsic(memmove)
extern "C" const void * __cdecl memchr(const void *buf, int chr, size_t buf_size);
#pragma intrinsic(memchr)

#pragma function(memset)
#pragma function(memcmp)
#pragma function(memcpy)
#pragma function(memmove)
#pragma function(memchr)

//#define _memcpy memcpy
//#define _memset memset

#define bzero(pvDst, count) memset(pvDst, 0, count)



static int
memcmp(const void *pv1, const void *pv2, size_t count) {
	register unsigned char *pbv1 = (unsigned char*)pv1, *pbv2 = (unsigned char*)pv2;

	if (0 == count || pv1 == pv2)
		return(0);

	while (-- count && (*pbv1) == (*pbv2)) {
		pbv1 ++;
		pbv2 ++;
	}
	return((*(pbv1)) - (*(pbv2)));
}



inline void *
memset(void *pvDst, int val, size_t count) {
	register unsigned char *pbDst = (unsigned char*)pvDst;
	register unsigned char bVal = (unsigned char)val;

	while (count --)
		(*(pbDst ++)) = bVal;

	return(pvDst);
}


static void *
memcpy(void *pvDst, const void *pvSrc, size_t count) {
	register unsigned char *pbDst = (unsigned char*)pvDst, *pbSrc = (unsigned char*)pvSrc;

	if (pvDst == pvSrc)
		return(pbDst);
	while (count --)
		(*(pbDst ++)) = (*(pbSrc ++));
	return(pvDst);
}

static void *
memmove(void *pvDst, const void *pvSrc, size_t count) {
	register unsigned char *pbDst = (unsigned char*)pvDst, *pbSrc = (unsigned char*)pvSrc;

	if (pvDst == pvSrc)
		return(pbDst);

	if (pvDst <= pvSrc || (char*)pvDst >= ((char*)pvSrc + count)) {
		// Non-Overlapping Buffers copy from lower addresses to higher addresses
		while (count --)
			(*(pbDst ++)) = (*(pbSrc ++));
	}else{// Overlapping Buffers copy from higher addresses to lower addresses
		pbDst += (count - 1);
		pbSrc += (count - 1);
		while (count --)
			(*(pbDst --)) = (*(pbSrc --));
	}
	return(pvDst);
}


static const void *
memchr(const void *buf, int chr, size_t buf_size) {
	register unsigned char *cur_pos = (unsigned char*)buf, *max_pos;

	if (NULL == buf || 0 == buf_size)
		return(NULL);

	max_pos = (unsigned char *)(cur_pos + buf_size);
	while (cur_pos < max_pos) {
		if ((*cur_pos) == chr)
			return(cur_pos);
		cur_pos ++;
	}

	return(NULL);
}







#endif // !defined(AFX_MEM_FUNCS__H__INCLUDED_)
